@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2008-2013 Sourcefire, Inc.
  *
  *  Author: aCaB <acab@clamav.net>
@@ -41,7 +41,7 @@
 #include "clamav.h"
 #include "default.h"
 
-// shared
+// common
 #include "output.h"
 #include "optparser.h"
 #include "misc.h"
@@ -49,7 +49,7 @@
 #include "connpool.h"
 #include "netcode.h"
 #include "clamfi.h"
-#include "whitelist.h"
+#include "allow_list.h"
 
 #ifndef _WIN32
 #include <sys/wait.h>
@@ -80,7 +80,7 @@ static void milter_exit(int sig)
     logg_close();
     cpool_free();
     localnets_free();
-    whitelist_free();
+    allow_list_free();
 }
 
 int main(int argc, char **argv)
@@ -96,7 +96,7 @@ int main(int argc, char **argv)
 
     sigset_t sigset;
     struct sigaction act;
-    const char * user_name = NULL;
+    const char *user_name = NULL;
 
     cl_initialize_crypto();
 
@@ -122,7 +122,7 @@ int main(int argc, char **argv)
         printf("\n");
         printf("                       Clam AntiVirus: Milter Mail Scanner %s\n", get_version());
         printf("           By The ClamAV Team: https://www.clamav.net/about.html#credits\n");
-        printf("           (C) 2020 Cisco Systems, Inc.\n");
+        printf("           (C) 2021 Cisco Systems, Inc.\n");
         printf("\n");
         printf("    %s [-c <config-file>]\n\n", argv[0]);
         printf("\n");
@@ -158,7 +158,7 @@ int main(int argc, char **argv)
     }
     free(pt);
 
-    if ((opt = optget(opts, "User"))->enabled){
+    if ((opt = optget(opts, "User"))->enabled) {
         user_name = opt->strarg;
     }
 
@@ -349,7 +349,7 @@ int main(int argc, char **argv)
         return 1;
     }
 
-    if ((opt = optget(opts, "Whitelist"))->enabled && whitelist_init(opt->strarg)) {
+    if (((opt = optget(opts, "Whitelist"))->enabled || (opt = optget(opts, "AllowList"))->enabled) && allow_list_init(opt->strarg)) {
         localnets_free();
         logg_close();
         optfree(opts);
@@ -358,7 +358,7 @@ int main(int argc, char **argv)
 
     if ((opt = optget(opts, "SkipAuthenticated"))->enabled && smtpauth_init(opt->strarg)) {
         localnets_free();
-        whitelist_free();
+        allow_list_free();
         logg_close();
         optfree(opts);
         return 1;
@@ -371,7 +371,7 @@ int main(int argc, char **argv)
         if (-1 == daemonize_parent_wait(user_name, logg_file)) {
             logg("!daemonize() failed\n");
             localnets_free();
-            whitelist_free();
+            allow_list_free();
             cpool_free();
             logg_close();
             optfree(opts);
@@ -410,7 +410,7 @@ int main(int argc, char **argv)
     if (!cp) {
         logg("!Failed to init the socket pool\n");
         localnets_free();
-        whitelist_free();
+        allow_list_free();
         logg_close();
         optfree(opts);
         return 1;
@@ -418,8 +418,8 @@ int main(int argc, char **argv)
 
     if ((opt = optget(opts, "PidFile"))->enabled) {
         FILE *fd;
-        mode_t old_umask = umask(0002);
-        int err = 0;
+        mode_t old_umask = umask(0022);
+        int err          = 0;
 
         if ((fd = fopen(opt->strarg, "w")) == NULL) {
             logg("!Can't save PID in file %s\n", opt->strarg);
@@ -434,14 +434,14 @@ int main(int argc, char **argv)
         umask(old_umask);
 
 #ifndef _WIN32
-        if (0 == err){
+        if (0 == err) {
             /*If the file has already been created by a different user, it will just be
              * rewritten by us, but not change the ownership, so do that explicitly.
              */
-            if (0 == geteuid()){
-                struct passwd * pw = getpwuid(0);
-                int ret = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
-                if (ret){
+            if (0 == geteuid()) {
+                struct passwd *pw = getpwuid(0);
+                int ret           = lchown(opt->strarg, pw->pw_uid, pw->pw_gid);
+                if (ret) {
                     logg("!Can't change ownership of PID file %s '%s'\n", opt->strarg, strerror(errno));
                     err = 1;
                 }
@@ -449,9 +449,9 @@ int main(int argc, char **argv)
         }
 #endif /*_WIN32*/
 
-        if (err){
+        if (err) {
             localnets_free();
-            whitelist_free();
+            allow_list_free();
             logg_close();
             optfree(opts);
             return 2;
@@ -460,7 +460,7 @@ int main(int argc, char **argv)
 
 #ifndef _WIN32
     dropPrivRet = drop_privileges(user_name, logg_file);
-    if (dropPrivRet){
+    if (dropPrivRet) {
         optfree(opts);
         return dropPrivRet;
     }
@@ -468,9 +468,9 @@ int main(int argc, char **argv)
     /* We have been daemonized, and initialization is done.  Signal
      * the parent process so that it can exit cleanly.
      */
-    if (parentPid != getpid()){ //we have been daemonized
+    if (parentPid != getpid()) { //we have been daemonized
         daemonize_signal_parent(parentPid);
-    }   
+    }
 #endif
 
     return smfi_main();
