@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2015-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2015-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  Authors: Mickey Sola
  *
@@ -37,7 +37,7 @@
 // libclamav
 #include "others.h"
 
-// shared
+// common
 #include "optparser.h"
 #include "output.h"
 
@@ -72,17 +72,19 @@ static int onas_scan(struct onas_scan_event *event_data, const char *fname, STAT
         switch (*ret_code) {
             case CL_EACCES:
             case CL_ESTAT:
-
-                logg("*ClamMisc: internal issue (daemon could not access directory/file %s)\n", fname);
+                logg("*ClamMisc: Scan issue; Daemon could not find or access: %s)\n", fname);
                 break;
                 /* TODO: handle other errors */
             case CL_EPARSE:
+                logg("~ClamMisc: Internal issue; Failed to parse reply from daemon: %s)\n", fname);
+                break;
             case CL_EREAD:
             case CL_EWRITE:
             case CL_EMEM:
             case CL_ENULLARG:
+            case CL_ERROR:
             default:
-                logg("~ClamMisc: internal issue (client failed to scan)\n");
+                logg("~ClamMisc: Unexpected issue; Daemon failed to scan: %s\n", fname);
         }
         if (retry_on_error) {
             logg("*ClamMisc: reattempting scan ... \n");
@@ -111,7 +113,7 @@ static cl_error_t onas_scan_safe(struct onas_scan_event *event_data, const char
 {
 
     int ret = 0;
-    int fd  = 0;
+    int fd  = -1;
 
 #if defined(HAVE_SYS_FANOTIFY_H)
     uint8_t b_fanotify;
@@ -349,7 +351,7 @@ void *onas_scan_worker(void *arg)
 		 * try to close it to resolve any potential lingering permissions event,
 		 * then move to cleanup */
         if (event_data->fmd) {
-            if (event_data->fmd->fd) {
+            if (event_data->fmd->fd >= 0) {
                 close(event_data->fmd->fd);
                 goto done;
             }
