@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2011-2013 Sourcefire, Inc.
  *
  *  Authors: aCaB
@@ -97,7 +97,7 @@ int cli_7unz(cli_ctx *ctx, size_t offset)
     archiveStream.s.Read    = FileInStream_fmap_Read;
     archiveStream.s.Seek    = FileInStream_fmap_Seek;
     archiveStream.s.curpos  = 0;
-    archiveStream.file.fmap = *ctx->fmap;
+    archiveStream.file.fmap = ctx->fmap;
 
     LookToRead_CreateVTable(&lookStream, False);
 
