@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2019-2020 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2019-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *
  *  EGG is an archive format created by ESTsoft used by their ALZip
  *  archiving software.
@@ -1590,7 +1590,7 @@ done:
     return status;
 }
 
-cl_error_t cli_egg_open(fmap_t* map, size_t sfx_offset, void** hArchive, char*** comments, uint32_t* nComments)
+cl_error_t cli_egg_open(fmap_t* map, void** hArchive, char*** comments, uint32_t* nComments)
 {
     cl_error_t status = CL_EPARSE;
     cl_error_t retval;
@@ -1610,7 +1610,7 @@ cl_error_t cli_egg_open(fmap_t* map, size_t sfx_offset, void** hArchive, char***
         goto done;
     }
     handle->map    = map;
-    handle->offset = sfx_offset;
+    handle->offset = 0;
 
     /*
      * 1st:
