@@ -1,13 +1,13 @@
 /*
- *  Copyright (C) 2013-2019 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Török Edvin
- * 
+ *
  *  Summary: Hash-table and -set data structures.
- * 
- *  Acknowledgements: hash32shift() is an implementation of Thomas Wang's 
- * 	                  32-bit integer hash function: 
+ *
+ *  Acknowledgements: hash32shift() is an implementation of Thomas Wang's
+ * 	                  32-bit integer hash function:
  * 	                  http://www.cris.com/~Ttwang/tech/inthash.htm
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@
 #include "clamav-types.h"
 #include "clamav-config.h"
 #include "mpool.h"
-typedef long cli_element_data;
+typedef size_t cli_element_data;
 
 /* define this for debugging/profiling purposes only, NOT in production/release code */
 #ifdef PROFILE_HASHTABLE
@@ -94,7 +94,7 @@ int cli_hashtab_store(const struct cli_hashtable *s, FILE *out);
 struct cli_htu32_element {
     uint32_t key;
     union {
-        unsigned long as_ulong;
+        size_t as_size_t;
         void *as_ptr;
     } data;
 };
