@@ -27,7 +27,7 @@
 #include "fmap.h"
 
 int cli_scanmacho(cli_ctx *ctx, struct cli_exe_info *fileinfo);
-int cli_machoheader(fmap_t *map, struct cli_exe_info *fileinfo);
+int cli_machoheader(cli_ctx *ctx, struct cli_exe_info *fileinfo);
 int cli_scanmacho_unibin(cli_ctx *ctx);
 int cli_unpackmacho(cli_ctx *ctx);
 
