@@ -581,9 +581,9 @@ static int get_filetype(const char *fname, int flags, int need_stat,
  *
  * @param fname         The file path
  * @param flags         CLI_FTW_* bitflag field
- * @param statbuf       [out] the stat metadata for the file.
- * @param stated        [out] 1 if statbuf contains stat info, 0 if not. -1 if there was a stat error.
- * @param ft            [out] will indicate if the file was skipped based on the file type.
+ * @param[out] statbuf  the stat metadata for the file.
+ * @param[out] stated   1 if statbuf contains stat info, 0 if not. -1 if there was a stat error.
+ * @param[out] ft       will indicate if the file was skipped based on the file type.
  * @param callback      the callback (E.g. function that may scan the file)
  * @param data          callback data
  * @return cl_error_t
@@ -1108,7 +1108,10 @@ char *cli_genfname(const char *prefix)
 
     fname = (char *)cli_calloc(len, sizeof(char));
     if (!fname) {
-        cli_dbgmsg("cli_genfname: out of memory\n");
+        cli_dbgmsg("cli_genfname: no memory left for fname\n");
+        if (NULL != sanitized_prefix) {
+            free(sanitized_prefix);
+        }
         return NULL;
     }
 
@@ -1129,7 +1132,10 @@ char *cli_genfname(const char *prefix)
 
     if (NULL == tmp) {
         free(fname);
-        cli_dbgmsg("cli_genfname: out of memory\n");
+        if (NULL != sanitized_prefix) {
+            free(sanitized_prefix);
+        }
+        cli_dbgmsg("cli_genfname: no memory left for cli_md5buff output\n");
         return NULL;
     }
 
@@ -1155,14 +1161,14 @@ char *cli_newfilepath(const char *dir, const char *fname)
 
     mdir = dir ? dir : cli_gettmpdir();
 
-    if (!fname) {
-        cli_dbgmsg("cli_newfilepath('%s'): out of memory\n", mdir);
+    if (NULL == fname) {
+        cli_dbgmsg("cli_newfilepath('%s'): fname argument must not be NULL\n", mdir);
         return NULL;
     }
 
     len      = strlen(mdir) + strlen(PATHSEP) + strlen(fname) + 1; /* mdir/fname\0 */
     fullpath = (char *)cli_calloc(len, sizeof(char));
-    if (!fullpath) {
+    if (NULL == fullpath) {
         cli_dbgmsg("cli_newfilepath('%s'): out of memory\n", mdir);
         return NULL;
     }
@@ -1174,9 +1180,16 @@ char *cli_newfilepath(const char *dir, const char *fname)
 
 cl_error_t cli_newfilepathfd(const char *dir, char *fname, char **name, int *fd)
 {
+    if (NULL == name || NULL == fname || NULL == fd) {
+        cli_dbgmsg("cli_newfilepathfd('%s'): invalid NULL arguments\n", dir);
+        return CL_EARG;
+    }
+
     *name = cli_newfilepath(dir, fname);
-    if (!*name)
+    if (!*name) {
+        cli_dbgmsg("cli_newfilepathfd('%s'): out of memory\n", dir);
         return CL_EMEM;
+    }
 
     *fd = open(*name, O_RDWR | O_CREAT | O_TRUNC | O_BINARY | O_EXCL, S_IRUSR | S_IWUSR);
     /*
@@ -1232,7 +1245,7 @@ cl_error_t cli_gentempfd(const char *dir, char **name, int *fd)
     return cli_gentempfd_with_prefix(dir, NULL, name, fd);
 }
 
-cl_error_t cli_gentempfd_with_prefix(const char *dir, char *prefix, char **name, int *fd)
+cl_error_t cli_gentempfd_with_prefix(const char *dir, const char *prefix, char **name, int *fd)
 {
     *name = cli_gentemp_with_prefix(dir, prefix);
     if (!*name)
@@ -1277,6 +1290,81 @@ int cli_regcomp(regex_t *preg, const char *pattern, int cflags)
     return cli_regcomp_real(preg, pattern, cflags);
 }
 
+#ifdef _WIN32
+cl_error_t cli_get_filepath_from_handle(HANDLE hFile, char **filepath)
+{
+    cl_error_t status               = CL_EARG;
+    char *evaluated_filepath        = NULL;
+    DWORD dwRet                     = 0;
+    WCHAR *long_evaluated_filepathW = NULL;
+    char *long_evaluated_filepathA  = NULL;
+    size_t evaluated_filepath_len   = 0;
+    cl_error_t conv_result;
+
+    if (NULL == filepath) {
+        cli_errmsg("cli_get_filepath_from_handle: Invalid args.\n");
+        goto done;
+    }
+
+    dwRet = GetFinalPathNameByHandleW((HANDLE)hFile, NULL, 0, VOLUME_NAME_DOS);
+    if (dwRet == 0) {
+        cli_dbgmsg("cli_get_filepath_from_handle: Failed to resolve handle\n");
+        status = CL_EOPEN;
+        goto done;
+    }
+
+    long_evaluated_filepathW = calloc(dwRet + 1, sizeof(WCHAR));
+    if (NULL == long_evaluated_filepathW) {
+        cli_errmsg("cli_get_filepath_from_filedesc: Failed to allocate %u bytes to store filename\n", dwRet + 1);
+        status = CL_EMEM;
+        goto done;
+    }
+
+    dwRet = GetFinalPathNameByHandleW((HANDLE)hFile, long_evaluated_filepathW, dwRet + 1, VOLUME_NAME_DOS);
+    if (dwRet == 0) {
+        cli_dbgmsg("cli_get_filepath_from_handle: Failed to resolve handle\n");
+        status = CL_EOPEN;
+        goto done;
+    }
+
+    if (0 == wcsncmp(L"\\\\?\\UNC", long_evaluated_filepathW, wcslen(L"\\\\?\\UNC"))) {
+        conv_result = cli_codepage_to_utf8(
+            long_evaluated_filepathW,
+            (wcslen(long_evaluated_filepathW)) * sizeof(WCHAR),
+            CODEPAGE_UTF16_LE,
+            &evaluated_filepath,
+            &evaluated_filepath_len);
+        if (CL_SUCCESS != conv_result) {
+            cli_errmsg("cli_get_filepath_from_handle: Failed to convert UTF16_LE filename to UTF8\n", dwRet + 1);
+            status = CL_EOPEN;
+            goto done;
+        }
+    } else {
+        conv_result = cli_codepage_to_utf8(
+            long_evaluated_filepathW + wcslen(L"\\\\?\\"),
+            (wcslen(long_evaluated_filepathW) - wcslen(L"\\\\?\\")) * sizeof(WCHAR),
+            CODEPAGE_UTF16_LE,
+            &evaluated_filepath,
+            &evaluated_filepath_len);
+        if (CL_SUCCESS != conv_result) {
+            cli_errmsg("cli_get_filepath_from_handle: Failed to convert UTF16_LE filename to UTF8\n", dwRet + 1);
+            status = CL_EOPEN;
+            goto done;
+        }
+    }
+
+    cli_dbgmsg("cli_get_filepath_from_handle: File path for handle %p is: %s\n", (void *)hFile, evaluated_filepath);
+    status    = CL_SUCCESS;
+    *filepath = evaluated_filepath;
+
+done:
+    if (NULL != long_evaluated_filepathW) {
+        free(long_evaluated_filepathW);
+    }
+    return status;
+}
+#endif
+
 cl_error_t cli_get_filepath_from_filedesc(int desc, char **filepath)
 {
     cl_error_t status        = CL_EARG;
@@ -1314,7 +1402,8 @@ cl_error_t cli_get_filepath_from_filedesc(int desc, char **filepath)
         goto done;
     }
 
-#elif __APPLE__
+#elif C_DARWIN
+
     char fname[PATH_MAX];
     memset(&fname, 0, PATH_MAX);
 
@@ -1337,65 +1426,21 @@ cl_error_t cli_get_filepath_from_filedesc(int desc, char **filepath)
     }
 
 #elif _WIN32
-    DWORD dwRet                     = 0;
-    intptr_t hFile                  = _get_osfhandle(desc);
-    WCHAR *long_evaluated_filepathW = NULL;
-    char *long_evaluated_filepathA  = NULL;
-    size_t evaluated_filepath_len   = 0;
-    cl_error_t conv_result;
+    intptr_t hFile = _get_osfhandle(desc);
+    cl_error_t handle_result;
 
     if (NULL == filepath) {
         cli_errmsg("cli_get_filepath_from_filedesc: Invalid args.\n");
         goto done;
     }
 
-    dwRet = GetFinalPathNameByHandleW((HANDLE)hFile, NULL, 0, VOLUME_NAME_DOS);
-    if (dwRet == 0) {
-        cli_dbgmsg("cli_get_filepath_from_filedesc: Failed to resolve filename for descriptor %d\n", desc);
+    handle_result = cli_get_filepath_from_handle((HANDLE)hFile, &evaluated_filepath);
+    if (CL_SUCCESS != handle_result) {
+        cli_errmsg("cli_get_filepath_from_filedesc: Failed to get file path from handle\n");
         status = CL_EOPEN;
         goto done;
     }
 
-    long_evaluated_filepathW = calloc(dwRet + 1, sizeof(WCHAR));
-    if (NULL == long_evaluated_filepathW) {
-        cli_errmsg("cli_get_filepath_from_filedesc: Failed to allocate %u bytes to store filename\n", dwRet + 1);
-        status = CL_EMEM;
-        goto done;
-    }
-
-    dwRet = GetFinalPathNameByHandleW((HANDLE)hFile, long_evaluated_filepathW, dwRet + 1, VOLUME_NAME_DOS);
-    if (dwRet == 0) {
-        cli_dbgmsg("cli_get_filepath_from_filedesc: Failed to resolve filename for descriptor %d\n", desc);
-        status = CL_EOPEN;
-        goto done;
-    }
-
-    if (0 == wcsncmp(L"\\\\?\\UNC", long_evaluated_filepathW, wcslen(L"\\\\?\\UNC"))) {
-        conv_result = cli_codepage_to_utf8(
-            long_evaluated_filepathW,
-            (wcslen(long_evaluated_filepathW)) * sizeof(WCHAR),
-            CODEPAGE_UTF16_LE,
-            &evaluated_filepath,
-            &evaluated_filepath_len);
-        if (CL_SUCCESS != conv_result) {
-            cli_errmsg("cli_get_filepath_from_filedesc: Failed to convert UTF16_LE filename to UTF8\n", dwRet + 1);
-            status = CL_EOPEN;
-            goto done;
-        }
-    } else {
-        conv_result = cli_codepage_to_utf8(
-            long_evaluated_filepathW + wcslen(L"\\\\?\\"),
-            (wcslen(long_evaluated_filepathW) - wcslen(L"\\\\?\\")) * sizeof(WCHAR),
-            CODEPAGE_UTF16_LE,
-            &evaluated_filepath,
-            &evaluated_filepath_len);
-        if (CL_SUCCESS != conv_result) {
-            cli_errmsg("cli_get_filepath_from_filedesc: Failed to convert UTF16_LE filename to UTF8\n", dwRet + 1);
-            status = CL_EOPEN;
-            goto done;
-        }
-    }
-
 #else
 
     cli_dbgmsg("cli_get_filepath_from_filedesc: No mechanism implemented to determine filename from file descriptor.\n");
@@ -1409,12 +1454,6 @@ cl_error_t cli_get_filepath_from_filedesc(int desc, char **filepath)
     *filepath = evaluated_filepath;
 
 done:
-
-#ifdef _WIN32
-    if (NULL != long_evaluated_filepathW) {
-        free(long_evaluated_filepathW);
-    }
-#endif
     return status;
 }
 
@@ -1423,7 +1462,9 @@ cl_error_t cli_realpath(const char *file_name, char **real_filename)
     char *real_file_path = NULL;
     cl_error_t status    = CL_EARG;
 #ifdef _WIN32
-    int desc = -1;
+    HANDLE hFile = INVALID_HANDLE_VALUE;
+#elif C_DARWIN
+    int fd = -1;
 #endif
 
     cli_dbgmsg("Checking realpath of %s\n", file_name);
@@ -1433,7 +1474,47 @@ cl_error_t cli_realpath(const char *file_name, char **real_filename)
         goto done;
     }
 
-#ifndef _WIN32
+#ifdef _WIN32
+
+    hFile = CreateFileA(file_name,                  // file to open
+                        GENERIC_READ,               // open for reading
+                        FILE_SHARE_READ,            // share for reading
+                        NULL,                       // default security
+                        OPEN_EXISTING,              // existing file only
+                        FILE_FLAG_BACKUP_SEMANTICS, // may be a directory
+                        NULL);                      // no attr. template
+    if (hFile == INVALID_HANDLE_VALUE) {
+        cli_warnmsg("Can't open file %s: %d\n", file_name, GetLastError());
+        status = CL_EOPEN;
+        goto done;
+    }
+
+    status = cli_get_filepath_from_handle(hFile, &real_file_path);
+
+#elif C_DARWIN
+
+    /* Using the filepath from filedesc method on macOS because
+       realpath will fail to check the realpath of a symbolic link if
+       the link doesn't point to anything.
+       Plus, we probably don't wan tot follow the link in this case anyways,
+       so this will check the realpath of the link, and not of the thing the
+       link points to. */
+    fd = open(file_name, O_RDONLY | O_SYMLINK);
+    if (fd == -1) {
+        char err[128];
+        cli_strerror(errno, err, sizeof(err));
+        if (errno == EACCES) {
+            status = CL_EACCES;
+        } else {
+            status = CL_EOPEN;
+        }
+        cli_dbgmsg("Can't open file %s: %s\n", file_name, err);
+        goto done;
+    }
+
+    status = cli_get_filepath_from_filedesc(fd, &real_file_path);
+
+#else
 
     real_file_path = realpath(file_name, NULL);
     if (NULL == real_file_path) {
@@ -1443,16 +1524,6 @@ cl_error_t cli_realpath(const char *file_name, char **real_filename)
 
     status = CL_SUCCESS;
 
-#else
-
-    if ((desc = safe_open(file_name, O_RDONLY | O_BINARY)) == -1) {
-        cli_warnmsg("Can't open file %s: %s\n", file_name, strerror(errno));
-        status = CL_EOPEN;
-        goto done;
-    }
-
-    status = cli_get_filepath_from_filedesc(desc, &real_file_path);
-
 #endif
 
     *real_filename = real_file_path;
@@ -1460,8 +1531,12 @@ cl_error_t cli_realpath(const char *file_name, char **real_filename)
 done:
 
 #ifdef _WIN32
-    if (-1 != desc) {
-        close(desc);
+    if (hFile != INVALID_HANDLE_VALUE) {
+        CloseHandle(hFile);
+    }
+#elif C_DARWIN
+    if (fd != -1) {
+        close(fd);
     }
 #endif
 
