@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2013-2022 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
+ *  Copyright (C) 2013-2021 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  *  Copyright (C) 2007-2013 Sourcefire, Inc.
  *
  *  Authors: Trog, Török Edvin
@@ -48,8 +48,7 @@
 
 int cli_check_mydoom_log(cli_ctx *ctx)
 {
-    uint32_t record[16];
-    const uint32_t *ptr;
+    const uint32_t *record;
     uint32_t check, key;
     fmap_t *map         = ctx->fmap;
     unsigned int blocks = map->len / (8 * 4);
@@ -60,24 +59,14 @@ int cli_check_mydoom_log(cli_ctx *ctx)
     if (blocks > 5)
         blocks = 5;
 
-    /*
-     * The following pointer might not be properly aligned. There there is
-     * memcmp() + memcpy() workaround to avoid performing an unaligned access
-     * while reading the uint32_t.
-     */
-    ptr = fmap_need_off_once(map, 0, 8 * 4 * blocks);
-    if (!ptr)
+    record = fmap_need_off_once(map, 0, 8 * 4 * blocks);
+    if (!record)
         return CL_CLEAN;
-
     while (blocks) { /* This wasn't probably intended but that's what the current code does anyway */
-        const uint32_t marker_ff = 0xffffffff;
-
-	if (!memcmp(&ptr[--blocks], &marker_ff, sizeof(uint32_t)))
+        if (record[--blocks] == 0xffffffff)
             return CL_CLEAN;
     }
 
-    memcpy(record, ptr, sizeof(record));
-
     key   = ~be32_to_host(record[0]);
     check = (be32_to_host(record[1]) ^ key) +
             (be32_to_host(record[2]) ^ key) +
